@@ -49,6 +49,8 @@ lj_entry_copy(LJEntry *e) {
 		newe->mood = g_strdup(e->mood);
 	if (e->music)
 		newe->music = g_strdup(e->music);
+	if (e->taglist)
+		newe->taglist = g_strdup(e->taglist);
 	if (e->pickeyword)
 		newe->pickeyword = g_strdup(e->pickeyword);
 	return newe;
@@ -59,12 +61,13 @@ lj_entry_free(LJEntry *e) {
 	g_free(e->event);
 	g_free(e->mood);
 	g_free(e->music);
+	g_free(e->taglist);
 	g_free(e->pickeyword);
 	g_free(e);
 }
 
-char*
-lj_get_summary(char *subject, char *event) {
+const char*
+lj_get_summary(const char *subject, const char *event) {
 #define SUMMARY_LENGTH 50
 	/* SUMMARY_LENGTH is in chars, not bytes.
 	 * UTF-8 chars can be up to 6 bytes, and then 
@@ -100,7 +103,7 @@ lj_get_summary(char *subject, char *event) {
 	return NULL;
 }
 
-char*
+const char*
 lj_entry_make_summary(LJEntry *entry) {
 	return lj_get_summary(entry->subject, entry->event);
 }
@@ -135,6 +138,8 @@ lj_entry_set_request_fields(LJEntry *entry, LJRequest *request) {
 		lj_request_add(request, "prop_current_moodid", "");
 	lj_request_add(request, 
 			"prop_current_music", entry->music ? entry->music : "");
+	lj_request_add(request, 
+			"prop_taglist", entry->taglist ? entry->taglist : "");
 	lj_request_add(request, 
 			"prop_picture_keyword", entry->pickeyword ? entry->pickeyword : "");
 	lj_request_add_int(request, "prop_opt_preformatted", entry->preformatted);
@@ -186,6 +191,12 @@ lj_entry_load_metadata(LJEntry *entry,
 			g_set_error(err, 0, 0, "Bad UTF-8 in current_music");
 			return FALSE;
 		}
+	} else if (strcmp(key, "taglist") == 0) {
+		entry->taglist = g_strdup(value);
+		if (!verify_utf8(&entry->taglist)) {
+			g_set_error(err, 0, 0, "Bad UTF-8 in taglist");
+			return FALSE;
+		}
 	} else if (strcmp(key, "picture_keyword") == 0) {
 		entry->pickeyword = g_strdup(value);
 		if (!verify_utf8(&entry->pickeyword)) {
@@ -210,17 +221,17 @@ lj_entry_load_metadata(LJEntry *entry,
 #ifdef HAVE_LIBXML
 static xmlNodePtr
 addtextchildenc(xmlDocPtr doc, xmlNodePtr parent, char *name, char *val) {
-	char *enc;
+	xmlChar *enc;
 	xmlNodePtr node;
-	enc = xmlEncodeEntitiesReentrant(doc, val);
-	node = xmlNewTextChild(parent, NULL, name, val);
+	enc = xmlEncodeEntitiesReentrant(doc, BAD_CAST val);
+	node = xmlNewTextChild(parent, NULL, BAD_CAST name, BAD_CAST val);
 	xmlFree(enc);
 	return node;
 }
 
 #define XML_ENTRY_META_GET(A)                                          \
-    if ((!strcmp(cur->name, #A))) {	                                   \
-        entry->A = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1); \
+    if ((!xmlStrcmp(cur->name, BAD_CAST #A))) {	                       \
+        entry->A = (char*)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1); \
     }
 
 #define XML_ENTRY_META_SET(A)                     \
@@ -230,14 +241,14 @@ addtextchildenc(xmlDocPtr doc, xmlNodePtr parent, char *name, char *val) {
 void*
 lj_entry_to_xml_node(LJEntry *entry, void* doc) {
 	xmlNodePtr root, node;
-	root = xmlNewDocNode(doc, NULL, "entry", NULL);
+	root = xmlNewDocNode(doc, NULL, BAD_CAST "entry", NULL);
 
 	/* do these in the order the fields are specified in livejournal.h,
 	 * so we don't miss anything. */
 	if (entry->itemid) {
 		char buf[10];
 		g_snprintf(buf, 10, "%d", entry->itemid);
-		xmlSetProp(root, "itemid", buf);
+		xmlSetProp(root, BAD_CAST "itemid", BAD_CAST buf);
 	}
 	XML_ENTRY_META_SET(subject);
 	XML_ENTRY_META_SET(event);
@@ -245,29 +256,30 @@ lj_entry_to_xml_node(LJEntry *entry, void* doc) {
 		if (entry->mood)
 			node = addtextchildenc(doc, root, "mood", entry->mood);
 		else
-			node = xmlNewChild(root, NULL, "mood", NULL);
+			node = xmlNewChild(root, NULL, BAD_CAST "mood", NULL);
 
 		if (entry->moodid) {
 			char buf[10];
 			g_snprintf(buf, 10, "%d", entry->itemid);
-			xmlSetProp(node, "itemid", buf);
+			xmlSetProp(node, BAD_CAST "itemid", BAD_CAST buf);
 		}
 	}
 	XML_ENTRY_META_SET(music);
+	XML_ENTRY_META_SET(taglist);
 	XML_ENTRY_META_SET(pickeyword);
 	if (entry->preformatted)
-		xmlNewChild(root, NULL, "preformatted", NULL);
+		xmlNewChild(root, NULL, BAD_CAST "preformatted", NULL);
 	if (entry->backdated)
-		xmlNewChild(root, NULL, "backdated", NULL);
+		xmlNewChild(root, NULL, BAD_CAST "backdated", NULL);
 	if (entry->comments) {
 		char *type = NULL;
-		node = xmlNewChild(root, NULL, "comments", NULL);
+		node = xmlNewChild(root, NULL, BAD_CAST "comments", NULL);
 		switch (entry->comments) {
 			case LJ_COMMENTS_DEFAULT: break;
 			case LJ_COMMENTS_NOEMAIL: type = "noemail"; break;
 			case LJ_COMMENTS_DISABLE: type = "disable"; break;
 		}
-		xmlSetProp(node, "type", type);
+		xmlSetProp(node, BAD_CAST "type", BAD_CAST type);
 	}
 	if (entry->time.tm_year) {
 		char *ljdate = lj_tm_to_ljdate(&entry->time);
@@ -279,10 +291,10 @@ lj_entry_to_xml_node(LJEntry *entry, void* doc) {
 		char *type = NULL, *mask = NULL;
 		lj_security_to_strings(&entry->security, &type, &mask);
 		if (type) {
-			node = xmlNewChild(root, NULL, "security", NULL);
-			xmlSetProp(node, "type", type);
+			node = xmlNewChild(root, NULL, BAD_CAST "security", NULL);
+			xmlSetProp(node, BAD_CAST "type", BAD_CAST type);
 			if (mask) {
-				xmlSetProp(node, "allowmask", mask);
+				xmlSetProp(node, BAD_CAST "allowmask", BAD_CAST mask);
 				g_free(mask);
 			}
 			g_free(type);
@@ -297,7 +309,7 @@ lj_entry_to_xml(LJEntry *entry) {
 	xmlDocPtr doc;
 	xmlNodePtr root;
 
-	doc = xmlNewDoc("1.0");
+	doc = xmlNewDoc(BAD_CAST "1.0");
 	root = lj_entry_to_xml_node(entry, doc);
 	xmlDocSetRootElement(doc, root);
 
@@ -348,7 +360,7 @@ lj_entry_new_from_filename(const char *filename, LJEntryFileType type,
 static LJEntry*
 lj_entry_new_from_result(LJResult *result, int i) {
 	LJEntry *entry;
-	char *itemid;
+	char *itemid, *anum;
 
 	itemid = lj_result_getf(result, "events_%d_itemid", i);
 	if (!itemid)
@@ -356,6 +368,8 @@ lj_entry_new_from_result(LJResult *result, int i) {
 
 	entry = lj_entry_new();
 	entry->itemid = atoi(itemid);
+	anum = lj_result_getf(result, "events_%d_anum", i);
+	if (anum) entry->anum = atoi(anum);
 	lj_ljdate_to_tm(lj_result_getf(result, "events_%d_eventtime", i), &entry->time);
 	entry->event = lj_urldecode(lj_result_getf(result, "events_%d_event", i));
 	verify_utf8(&entry->event);
@@ -444,53 +458,55 @@ lj_entries_new_from_result(LJResult *result, int count,
 #ifdef HAVE_LIBXML
 static void
 lj_entry_load_from_xml_node(LJEntry *entry, xmlDocPtr doc, xmlNodePtr node) {
-	char *itemid;
+	xmlChar *itemid;
 	xmlNodePtr cur;
 
-	if ((itemid = xmlGetProp(node, "itemid")) != NULL) {
-		entry->itemid = atoi(itemid);
+	if ((itemid = xmlGetProp(node, BAD_CAST "itemid")) != NULL) {
+		entry->itemid = atoi((char*)itemid);
 		xmlFree(itemid);
 	}
 	cur = node->xmlChildrenNode;
 	while (cur != NULL) {
 		XML_ENTRY_META_GET(subject)
 		else XML_ENTRY_META_GET(event)
-		else if (strcmp(cur->name, "mood") == 0) {
-			char *id;
-			entry->mood = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+		else if (xmlStrcmp(cur->name, BAD_CAST "mood") == 0) {
+			xmlChar *id;
+			entry->mood = (char*)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
 			if (entry->mood && strlen(entry->mood) == 0) {
 				g_free(entry->mood); entry->mood = NULL;
 			}
-			if ((id = xmlGetProp(cur, "type")) != NULL) {
-				entry->moodid = atoi(id);
+			if ((id = xmlGetProp(cur, BAD_CAST "type")) != NULL) {
+				entry->moodid = atoi((char*)id);
 				xmlFree(id);
 			}
 		}
 		else XML_ENTRY_META_GET(music)
+		else XML_ENTRY_META_GET(taglist)
 		else XML_ENTRY_META_GET(pickeyword)
-		else if (strcmp(cur->name, "preformatted") == 0) {
+		else if (xmlStrcmp(cur->name, BAD_CAST "preformatted") == 0) {
 			entry->preformatted = TRUE;
-		} else if (strcmp(cur->name, "backdated") == 0) {
+		} else if (xmlStrcmp(cur->name, BAD_CAST "backdated") == 0) {
 			entry->backdated = TRUE;
-		} else if (strcmp(cur->name, "comments") == 0) {
-			char *type;
-			if ((type = xmlGetProp(cur, "type")) != NULL) {
-				if (strcmp(type, "noemail") == 0) {
+		} else if (xmlStrcmp(cur->name, BAD_CAST "comments") == 0) {
+			xmlChar *type;
+			if ((type = xmlGetProp(cur, BAD_CAST "type")) != NULL) {
+				if (xmlStrcmp(type, BAD_CAST "noemail") == 0) {
 					entry->comments = LJ_COMMENTS_NOEMAIL;
-				} else if (strcmp(type, "disable") == 0) {
+				} else if (xmlStrcmp(type, BAD_CAST "disable") == 0) {
 					entry->comments = LJ_COMMENTS_DISABLE;
 				}
 				xmlFree(type);
 			}
-		} else if (strcmp(cur->name, "time") == 0) {
-			char *date = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			lj_ljdate_to_tm(date, &entry->time);
+		} else if (xmlStrcmp(cur->name, BAD_CAST "time") == 0) {
+			xmlChar *date = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			lj_ljdate_to_tm((char*)date, &entry->time);
 			g_free(date);
-		} else if (strcmp(cur->name, "security") == 0) {
-			char *type, *mask;
-			type = xmlGetProp(cur, "type");
-			mask = xmlGetProp(cur, "allowmask");
-			lj_security_from_strings(&entry->security, type, mask);
+		} else if (xmlStrcmp(cur->name, BAD_CAST "security") == 0) {
+			xmlChar *type, *mask;
+			type = xmlGetProp(cur, BAD_CAST "type");
+			mask = xmlGetProp(cur, BAD_CAST "allowmask");
+			lj_security_from_strings(&entry->security,
+			                         (char*)type, (char*)mask);
 			xmlFree(type);
 			xmlFree(mask);
 		}
@@ -586,6 +602,7 @@ rfc822_load_entry(const char *key, const char *val, LJEntry *entry) {
 	RFC822_GET(subject)
 	else RFC822_GET(mood) /* XXX id */
 	else RFC822_GET(music)
+	else RFC822_GET(taglist)
 	else RFC822_GET(pickeyword)
 	else if (g_ascii_strcasecmp(key, "time") == 0) {
 		if (entry) lj_ljdate_to_tm(val, &entry->time);
@@ -722,6 +739,7 @@ lj_entry_to_rfc822(LJEntry *entry, gboolean includeempty) {
 	append_field(str, "Subject", entry->subject, includeempty);
 	append_field(str, "Mood", entry->mood, includeempty);
 	append_field(str, "Music", entry->music, includeempty);
+	append_field(str, "TagList", entry->taglist, includeempty);
 	append_field(str, "PicKeyword", entry->pickeyword, includeempty);
 	g_string_append(str, "\n");
 	if (entry->event)
