@@ -19,13 +19,13 @@ typedef struct _Sync {
 	int total;
 
 	gboolean sync_complete;
-	GSList *syncitems;
+	GHashTable *syncitems;
 
 	GError **err;
 
 	LJPutLastSyncCallback put_lastsync;
 	LJRunVerbCallback run_verb;
-	LJPutEntryCallback put_entry;
+	LJPutEntriesCallback put_entries;
 	LJSyncProgressCallback sync_progress;
 	gpointer user_data;
 } Sync;
@@ -45,14 +45,20 @@ entry_date_compare_func(const void *a, const void *b, void *unused) {
 	return 0;
 }
 
+static void
+sync_report_progress(Sync *sync, LJSyncProgress progress) {
+	sync->sync_progress(sync->user_data, progress,
+	                    sync->synced, sync->total, sync->lastsync);
+}
+
 static time_t
 run_one_getevents(Sync *sync, GSList **warnings) {
 	LJGetEventsSync *getevents;
-	int lastitemid, i;
+	int i;
 	time_t lastsync = 0;
 
-	getevents = lj_getevents_sync_new(sync->user,
-			sync->usejournal, sync->lastsync);
+	getevents = lj_getevents_sync_new(sync->user, sync->usejournal,
+	                                  sync->lastsync);
 	if (!sync->run_verb(sync->user_data, (LJVerb*)getevents, sync->err)) {
 		lj_getevents_sync_free(getevents, TRUE);
 		return 0;
@@ -63,35 +69,26 @@ run_one_getevents(Sync *sync, GSList **warnings) {
 		return 0;
 	}
 
-	lastitemid = getevents->entries[getevents->entry_count-1]->itemid;
-	/*g_print("lastsync %s : got %d entries\n",
-			sync->lastsync ? sync->lastsync : "(none)", getevents->entry_count);*/
+	/* g_print("lastsync %s : got %d entries\n",
+			sync->lastsync ? sync->lastsync : "(none)", getevents->entry_count);
+	*/
 
-	/* pop off all the syncitems for the entries we have now received,
-	 * also scanning for the latest time we managed to sync.
-	 * this process is O(e*s), e=entry_count, s=sync_count,
-	 * but in practice we find the entries fairly early so s is small. */
+	/* remove off all the syncitems for the entries we have now received,
+	 * also scanning for the latest time we managed to sync. */
 	for (i = 0; i < getevents->entry_count; i++) {
 		int id = getevents->entries[i]->itemid;
-		GSList *last = NULL, *cur = sync->syncitems;
+		time_t itemtime = GPOINTER_TO_INT(
+				g_hash_table_lookup(sync->syncitems, GINT_TO_POINTER(id)));
 
-		while (cur && ((LJSyncItem*)cur->data)->id != id) {
-			last = cur; cur = cur->next;
-		}
-
-		if (cur) {
-			LJSyncItem *item = (LJSyncItem*)cur->data;
+		if (itemtime) {
 			sync->synced++;
 
 			/* update latest time. */
-			if (item->time > lastsync)
-				lastsync = item->time;
+			if (itemtime > lastsync)
+				lastsync = itemtime;
 
 			/* pop this off the syncitems list. */
-			if (last) last->next = cur->next;
-			else sync->syncitems = cur->next;
-			g_free(item);
-			g_slist_free_1(cur);
+			g_hash_table_remove(sync->syncitems, GINT_TO_POINTER(id));
 		}
 	}
 
@@ -101,9 +98,7 @@ run_one_getevents(Sync *sync, GSList **warnings) {
 	g_qsort_with_data(getevents->entries, getevents->entry_count,
 	                  sizeof(LJEntry*), entry_date_compare_func, NULL);
 	/* and finally, put these to disk. */
-	for (i = 0; i < getevents->entry_count; i++) {
-		sync->put_entry(sync->user_data, getevents->entries[i], sync->err);
-	}
+	sync->put_entries(sync->user_data, getevents->entries, getevents->entry_count, sync->err);
 
 	lj_getevents_sync_free(getevents, TRUE);
 
@@ -115,18 +110,16 @@ run_one_syncitems(Sync *sync) {
 	LJSyncItems *syncitems;
 
 	syncitems = lj_syncitems_new(sync->user, sync->usejournal,
-	                             sync->lastsync);
-	syncitems->items = sync->syncitems;
+	                             sync->lastsync, sync->syncitems);
 	if (!sync->run_verb(sync->user_data, (LJVerb*)syncitems, sync->err)) {
 		lj_syncitems_free(syncitems, TRUE);
-		return 0;
+		sync->syncitems = NULL;
+		return FALSE;
 	}
 
 	if (!sync->total)
 		sync->total = syncitems->total;
-
-	if (syncitems->count == syncitems->total)
-		sync->sync_complete = TRUE;
+	sync->synced = sync->total - (syncitems->total - syncitems->count);
 
 	g_free(sync->lastsync);
 	sync->lastsync = g_strdup(syncitems->lastsync);
@@ -139,55 +132,25 @@ run_one_syncitems(Sync *sync) {
 
 static gboolean
 run_syncitems(Sync *sync) {
-	gboolean sentfirst = FALSE;
-	int itemcount;
 	do {
 		if (!run_one_syncitems(sync))
 			return FALSE;
-		if (!sentfirst) {
-			sync->sync_progress(sync->user_data, LJ_SYNC_PROGRESS_ITEMS,
-					0, sync->total, sync->lastsync);
-			sentfirst = TRUE;
-		}
-		itemcount = g_slist_length(sync->syncitems);
-		if (itemcount > 0)
-			sync->sync_progress(sync->user_data, LJ_SYNC_PROGRESS_ITEMS,
-					itemcount, sync->total, sync->lastsync);
-	} while (!sync->sync_complete);
-	
+		sync_report_progress(sync, LJ_SYNC_PROGRESS_ITEMS);
+	} while (sync->synced < sync->total);
+
 	return TRUE;
 }
 
 static gboolean
 run_getevents(Sync *sync, GSList **warnings) {
 	time_t lastsync;
-	int entrycount = 0;
-	GSList *cur = sync->syncitems, *last = NULL;
 
-	/* strip off non-entry syncitems and
-	 * count how many entries we have to sync. */
-	while (cur) {
-		LJSyncItem *item = (LJSyncItem*)cur->data;
-		if (item->type != LJ_SYNCITEM_TYPE_ENTRY) {
-			GSList *next = cur->next;
+	sync->synced = 0;
+	sync->total = g_hash_table_size(sync->syncitems);
 
-			/* pop this off the syncitems list. */
-			if (last) last->next = cur->next;
-			else sync->syncitems = cur->next;
-			g_free(item);
-			g_slist_free_1(cur);
-			cur = next;
-		} else {
-			last = cur; cur = cur->next;
-			entrycount++;
-		}
-	}
+	sync_report_progress(sync, LJ_SYNC_PROGRESS_ENTRIES);
 
-	sync->sync_progress(sync->user_data, LJ_SYNC_PROGRESS_ENTRIES,
-			0, entrycount, sync->lastsync);
-
-	/* now sync them. */
-	while (sync->syncitems) {
+	while (g_hash_table_size(sync->syncitems) > 0) {
 		lastsync = run_one_getevents(sync, warnings);
 		if (lastsync <= 0)
 			return FALSE;
@@ -200,8 +163,7 @@ run_getevents(Sync *sync, GSList **warnings) {
 		sync->lastsync = lj_tm_to_ljdate(gmtime(&lastsync));
 		sync->put_lastsync(sync->user_data, sync->lastsync, sync->err);
 
-		sync->sync_progress(sync->user_data, LJ_SYNC_PROGRESS_ENTRIES,
-				sync->synced, entrycount, sync->lastsync);
+		sync_report_progress(sync, LJ_SYNC_PROGRESS_ENTRIES);
 	}
 
 	return TRUE;
@@ -210,7 +172,7 @@ run_getevents(Sync *sync, GSList **warnings) {
 gboolean
 lj_sync_run(LJUser *user, const char *usejournal,
             const char *lastsync, LJPutLastSyncCallback put_lastsync_cb,
-            LJRunVerbCallback run_verb_cb, LJPutEntryCallback put_entry_cb,
+            LJRunVerbCallback run_verb_cb, LJPutEntriesCallback put_entries_cb,
             LJSyncProgressCallback sync_progress_cb,
 			gpointer user_data, GSList **warnings, GError **err) {
 	Sync st = {0}, *sync = &st;
@@ -221,7 +183,7 @@ lj_sync_run(LJUser *user, const char *usejournal,
 	sync->lastsync = lastsync ? g_strdup(lastsync) : NULL;
 	sync->put_lastsync = put_lastsync_cb;
 	sync->run_verb = run_verb_cb;
-	sync->put_entry = put_entry_cb;
+	sync->put_entries = put_entries_cb;
 	sync->sync_progress = sync_progress_cb;
 	sync->user_data = user_data;
 	sync->err = err;
@@ -234,10 +196,7 @@ lj_sync_run(LJUser *user, const char *usejournal,
 	}
 
 	g_free(sync->lastsync);
-	/* there shouldn't be any leftover syncitems in the list,
-	 * but it's good to be safe. */
-	g_slist_foreach(sync->syncitems, (GFunc)g_free, NULL);
-	g_slist_free(sync->syncitems);
+	g_hash_table_destroy(sync->syncitems);
 	return success;
 }
 
