@@ -9,7 +9,7 @@
 #ifdef HAVE_GTK
 #include "gtk-all.h"
 #else
-#include <glib.h>
+#include "glib-all.h"
 #endif
 
 #ifdef HAVE_REGEX_H
@@ -21,6 +21,8 @@
 #include <unistd.h>
 #endif
 
+#include <string.h>
+
 #include <livejournal/consolecommand.h>
 #include <livejournal/login.h>
 
@@ -29,7 +31,6 @@
 #include "conf_xml.h"
 #include "journalstore.h"
 #include "network.h"
-#include "util.h"
 #include "checkfriends.h"
 #include "sync.h"
 
@@ -321,7 +322,7 @@ do_offline_cat(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[]) {
 				_("Specify at least one itemid or 'latest'.\n"));
 		exit(EXIT_FAILURE);
 	}
-	if (!(js = journal_store_new(acc, &err))) {
+	if (!(js = journal_store_open(acc, FALSE, &err))) {
 		fprintf(stderr,
 				"%s", err->message);
 		g_error_free(err);
@@ -428,7 +429,7 @@ do_offline_grep(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[]) {
 		g_printerr(_("Specify a regexp to match\n"));
 		exit(EXIT_FAILURE);
 	}
-	if (!(js = journal_store_new(acc, &err))) {
+	if (!(js = journal_store_open(acc, FALSE, &err))) {
 		g_printerr("%s", err->message);
 		g_error_free(err);
 		exit(EXIT_FAILURE);
@@ -498,7 +499,7 @@ do_offline_summary(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[])
 	GETOPT_LOOP_END
 	command_dispatch(cmdline, NULL, _(OFFLINE_SUMMARY_HELP_TEXT), argc, argv);
 
-	if (!(js = journal_store_new(acc, &err))) {
+	if (!(js = journal_store_open(acc, FALSE, &err))) {
 		fprintf(stderr,
 				"%s", err->message);
 		g_error_free(err);
