@@ -4,7 +4,8 @@
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
  */
 
-#include <config.h>
+#include "config.h"
+
 #include "gtk-all.h"
 
 #include <stdio.h>
@@ -26,7 +27,7 @@
 
 #include "network.h"
 #include "conf.h"
-#include "util.h"
+#include "util-gtk.h"
 #include "login.h"
 #include "jamdoc.h"
 #include "jamview.h"
@@ -98,13 +99,17 @@ jam_save_as_file(JamWin *jw) {
 	GError *err = NULL;
 	gboolean ret = FALSE;
 	
-	filesel = jam_filesel_new(_("Save Entry As"), GTK_WINDOW(jw));
+	filesel = gtk_file_chooser_dialog_new(_("Save Entry As"), GTK_WINDOW(jw),
+			GTK_FILE_CHOOSER_ACTION_SAVE,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+			NULL);
 
-	while (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK) {
+	while (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
 		const gchar *filename;
 		struct stat sbuf;
-		filename = gtk_file_selection_get_filename(
-				GTK_FILE_SELECTION(filesel));
+		filename = gtk_file_chooser_get_filename(
+				GTK_FILE_CHOOSER(filesel));
 		if (stat(filename, &sbuf) == 0) {
 			if (!jam_confirm(GTK_WINDOW(filesel), _("Save Entry As"),
 						_("File already exists!  Overwrite?")))
@@ -491,7 +496,11 @@ jam_open_entry(JamWin *jw) {
 
 	if (!jam_confirm_lose_entry(jw)) return;
 	
-	filesel = jam_filesel_new(_("Open Entry"), GTK_WINDOW(jw));
+	filesel = gtk_file_chooser_dialog_new(_("Open Entry"), GTK_WINDOW(jw),
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			NULL);
 
 	filetype = gtk_option_menu_new();
 	menu = gtk_menu_new();
@@ -508,17 +517,16 @@ jam_open_entry(JamWin *jw) {
 	gtk_box_pack_start(GTK_BOX(hbox), filetype, TRUE, TRUE, 0);
 	gtk_widget_show_all(hbox);
 
-	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(filesel)->main_vbox),
-			hbox, FALSE, FALSE, 0);
+	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(filesel), hbox);
 	
-	while (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK) {
+	while (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
 		const gchar *filename;
 		GError *err = NULL;
 		int op = gtk_option_menu_get_history(GTK_OPTION_MENU(filetype));
 		LJEntryFileType type = filetypes[op].type;
 
-		filename = gtk_file_selection_get_filename(
-				GTK_FILE_SELECTION(filesel));
+		filename = gtk_file_chooser_get_filename(
+				GTK_FILE_CHOOSER(filesel));
 
 		if (!jam_doc_load_file(jw->doc, filename, type, &err)) {
 			jam_warning(GTK_WINDOW(filesel), err->message);
