@@ -6,11 +6,14 @@
 
 #include "config.h"
 
-#include <glib.h>
-#include "util.h"
+#include "glib-all.h"
+
+#include <string.h>
+
 #include "draftstore.h"
 #include "jamdoc.h"
 #include "account.h"
+#include "conf.h"
 
 #ifdef HAVE_GTK
 #include "get_cmd_out.h"
@@ -238,6 +241,14 @@ void
 jam_doc_set_music(JamDoc *doc, const char *music) {
 	string_replace(&doc->entry->music, music ? g_strdup(music) : NULL);
 }
+const char *
+jam_doc_get_taglist(JamDoc *doc) {
+	return doc->entry->taglist;
+}
+void
+jam_doc_set_taglist(JamDoc *doc, const char *taglist) {
+	string_replace(&doc->entry->taglist, taglist ? g_strdup(taglist) : NULL);
+}
 LJSecurity
 jam_doc_get_security(JamDoc *doc) {
 	return doc->entry->security;
