@@ -7,7 +7,6 @@
 #include "config.h"
 
 #include "gtk-all.h"
-#include <glib.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -18,7 +17,7 @@
 #include "jam.h"
 #include "friends.h"
 #include "history.h"
-#include "util.h"
+#include "util-gtk.h"
 #include "spawn.h"
 #include "security.h"
 #include "icons.h"
@@ -56,6 +55,7 @@ enum {
 	ACTION_VIEW_MOOD,
 	ACTION_VIEW_PICTURE,
 	ACTION_VIEW_MUSIC,
+	ACTION_VIEW_TAGS,
 	ACTION_VIEW_PREFORMATTED,
 	ACTION_VIEW_DATESEL,
 	ACTION_VIEW_COMMENTS,
@@ -405,7 +405,7 @@ static GtkItemFactoryEntry menu_items[] = {
 	0, NULL },
 {    "/Insert/---",               NULL, NULL, 0, "<Separator>" },
 { N_("/Insert/_Link..."),         "<ctl>L",      menu_make_link },
-{ N_("/Insert/_Image..."),        "<ctl><alt>I", menu_insert_image },
+{ N_("/Insert/_Image..."),        NULL,          menu_insert_image },
 { N_("/Insert/_Journal Link..."), "<ctl><alt>L", menu_make_journal_link },
 { N_("/Insert/lj-_cut..."),       "<ctl><alt>X", menu_ljcut, 0, NULL },
 
@@ -414,6 +414,7 @@ static GtkItemFactoryEntry menu_items[] = {
 { N_("/View/Entry _Date"),        NULL, menu_view_cb, ACTION_VIEW_DATESEL,      "<CheckItem>" },
 { N_("/View/_Mood"),              NULL, menu_view_cb, ACTION_VIEW_MOOD,         "<CheckItem>" },
 { N_("/View/_Picture"),           NULL, menu_view_cb, ACTION_VIEW_PICTURE,      "<CheckItem>" },
+{ N_("/View/_Tags"),              NULL, menu_view_cb, ACTION_VIEW_TAGS,         "<CheckItem>" },
 { N_("/View/M_usic"),             NULL, menu_view_cb, ACTION_VIEW_MUSIC,        "<CheckItem>" },
 { N_("/View/_Preformatted"),      NULL, menu_view_cb, ACTION_VIEW_PREFORMATTED, "<CheckItem>" },
 { N_("/View/_Comments"),          NULL, menu_view_cb, ACTION_VIEW_COMMENTS,     "<CheckItem>" },
