@@ -5,9 +5,12 @@
  */
 
 #include "config.h"
+
 #include "gtk-all.h"
 
-#include "util.h"
+#include <string.h>
+
+#include "util-gtk.h"
 #ifdef HAVE_GTKHTML
 #include "preview.h"
 #endif
@@ -66,7 +69,7 @@ month_changed_cb(GtkCalendar *calendar, OfflineUI *oui) {
 }
 
 static void
-get_entries_cb(int itemid, time_t etime, char *summary, LJSecurity *sec, gpointer data) {
+get_entries_cb(int itemid, time_t etime, const char *summary, LJSecurity *sec, gpointer data) {
 	GtkTreeIter iter;
 	OfflineUI *oui = (OfflineUI*)data;
 	GdkPixbuf *pb;
@@ -204,7 +207,7 @@ select_entry(OfflineUI *oui, int itemid) {
 
 	etime = journal_store_lookup_entry_time(oui->journalstore, itemid);
 
-	etm = localtime(&etime);
+	etm = gmtime(&etime);
 	gtk_calendar_get_date(GTK_CALENDAR(oui->cal), &year, &mon, &day);
 
 	gtk_calendar_freeze(GTK_CALENDAR(oui->cal));
@@ -252,8 +255,7 @@ move_relative(OfflineUI *oui, int dir) {
 
 	if (itemid) { /* selection exists */
 		found = journal_store_find_relative(oui->journalstore, itemid,
-				&itemid, &etime,
-				dir, NULL);
+				&itemid, dir, NULL);
 	} else {     /* some more cajoling necessary */
 		guint year, month, day;
 		struct tm when_tm = {0};
@@ -263,9 +265,7 @@ move_relative(OfflineUI *oui, int dir) {
 		when_tm.tm_mday = day;
 		
 		found = journal_store_find_relative_by_time(oui->journalstore,
-				mktime(&when_tm),
-				&itemid, &etime,
-				dir, NULL);
+				timegm(&when_tm), &itemid, dir, NULL);
 	}
 	if (found)
 		select_entry(oui, itemid);
@@ -319,6 +319,7 @@ static gboolean
 search_case_cb(const char *str, gpointer data) {
 	char *down;
 	gboolean ret;
+	if (!str) return FALSE;
 	down = g_utf8_strdown(str, -1);
 	ret = (strstr(down, data) != NULL);
 	g_free(down);
@@ -327,7 +328,7 @@ search_case_cb(const char *str, gpointer data) {
 
 static gboolean
 search_cb(const char *str, gpointer data) {
-	return strstr(str, data) != NULL;
+	return str && strstr(str, data) != NULL;
 }
 
 static void
@@ -339,12 +340,12 @@ find_cb(GtkWidget *button, OfflineUI *oui) {
 
 	gtk_list_store_clear(oui->store);
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(oui->searchcase))) {
-		journal_store_search(oui->journalstore,
+		journal_store_scan(oui->journalstore,
 				search_cb, (gpointer)search,
 				get_entries_cb, oui);
 	} else {
 		char *lsearch = g_utf8_strdown(search, -1);
-		journal_store_search(oui->journalstore,
+		journal_store_scan(oui->journalstore,
 				search_case_cb, lsearch,
 				get_entries_cb, oui);
 		g_free(lsearch);
@@ -619,7 +620,7 @@ offline_dlg_run(GtkWindow *parent, JamAccount *acc) {
 	gboolean run = TRUE;
 
 	oui = offlineui_new();
-	OFFLINEUI(oui)->journalstore = journal_store_new(acc, NULL);
+	OFFLINEUI(oui)->journalstore = journal_store_open(acc, FALSE, NULL);
 	OFFLINEUI(oui)->account = acc;
 	month_changed_cb(GTK_CALENDAR(OFFLINEUI(oui)->cal), OFFLINEUI(oui));
 	day_selected_cb (GTK_CALENDAR(OFFLINEUI(oui)->cal), OFFLINEUI(oui));
